@@ -1,8 +1,8 @@
 import re
-import os
 
 import xml.sax.handler
 import rope.base.pynames
+import rope.base.pyobjects
 
 import gobject
 
@@ -47,6 +47,11 @@ class GladeName(rope.base.pynames.PyName):
         return self.module, self.line
 
 
+class GladeFunction(rope.base.pyobjects.AbstractFunction):
+    def __init__(self):
+        rope.base.pyobjects.AbstractFunction.__init__(self)
+
+
 class PyGtkHintProvider(HintProvider):
     def __init__(self, project):
         super(PyGtkHintProvider, self).__init__(project)
@@ -73,16 +78,18 @@ class PyGtkHintProvider(HintProvider):
             type = self.get_type(self.get_pygtk_class_name(cls))
             attrs[id] = GladeName(type.get_object(), ResourceAsModule(glade_resource), line)
 
-        self.cache[scope_path] = attrs
-
-        for name, (cls, signal) in handler.signals.iteritems():
+        for name, (cls, signal, line) in handler.signals.iteritems():
             self.handlers.setdefault(scope_path, {})[name] = cls, signal
+            attrs[name] = GladeName(GladeFunction(), ResourceAsModule(glade_resource), line)
 
-        self.gtk_aware_classes[scope_path] = glade_file
+        self.cache[scope_path] = attrs
+        self.processed_files[glade_file] = True
 
     def get_glade_file_for_class(self, scope_path, pyclass):
+        project = pyclass.get_module().resource.project
         try:
-            return self.gtk_aware_classes[scope_path]
+            path = self.gtk_aware_classes[scope_path]
+            return project.get_resource(path)
         except KeyError:
             pass
 
@@ -92,49 +99,38 @@ class PyGtkHintProvider(HintProvider):
             if match:
                 filename = match.group(1).strip()
                 if filename.startswith('/'):
-                    return filename[1:]
+                    return project.get_resource(filename[1:])
                 else:
-                    return os.path.join(os.path.dirname(pyclass.get_module().resource.path), filename)
+                    return pyclass.get_module().resource.parent.get_child(filename)
 
         return None
 
-    def get_class_attributes(self, scope_path, pyclass, attrs):
+    def get_attributes(self, scope_path, pyclass, orig_attrs):
         attrs = {}
         glade_file = self.get_glade_file_for_class(scope_path, pyclass)
         if glade_file:
-            glade_resource = pyclass.get_module().resource.project.get_file(glade_file)
-            self.process_glade(scope_path, glade_resource)
+            self.process_glade(scope_path, glade_file)
             for k, v in self.cache[scope_path].iteritems():
-                attrs[k] = v
+                if k not in orig_attrs:
+                    attrs[k] = v
 
         return attrs
 
     def add_class(self, scope, glade_file):
         self.gtk_aware_classes[scope] = glade_file
 
-    def get_function_param_type(self, pyfunc, name):
-        """Resolve function's parameters types from doc string
+    def get_function_params(self, scope_path, pyfunc):
+        """:type pyfunc: rope.base.pyobjectsdef.PyFunction"""
 
-        :type pyfunc: rope.base.pyobjectsdef.PyFunction
-        """
-        scope_path = self.get_scope_path(pyfunc.get_scope())
-        try:
-            params = self.func_cache[scope_path]
-        except KeyError:
-            pyclass = pyfunc.parent
-            class_scope = get_attribute_scope_path(pyclass)
-            glade_file = self.get_glade_file_for_class(scope_path, pyclass)
+        pyclass = pyfunc.parent
+        scope_path = get_attribute_scope_path(pyclass)
+        glade_file = self.get_glade_file_for_class(scope_path, pyclass)
 
-            if glade_file:
-                glade_resource = pyclass.get_module().resource.project.get_file(glade_file)
-                self.process_glade(scope_path, glade_resource)
-                params = self.get_params_for_handler(class_scope, pyfunc)
-            else:
-                params = {}
-
-            self.func_cache[scope_path] = params
-
-        return params.get(name, None)
+        if glade_file:
+            self.process_glade(scope_path, glade_file)
+            return self.get_params_for_handler(scope_path, pyfunc)
+        else:
+            return {}
 
     def get_params_for_handler(self, class_scope, pyfunc):
         """:type pyfunc: rope.base.pyobjectsdef.PyFunction"""
@@ -145,24 +141,24 @@ class PyGtkHintProvider(HintProvider):
 
         attrs = {}
 
+        idx = 0
         names = pyfunc.get_param_names(False)
         if pyfunc.get_kind() in ('method', 'classmethod'):
             names = names[1:]
+            idx += 1
 
         if names:
-            attrs[names[0]] = self.get_type(self.get_pygtk_class_name(cls)).get_object()
+            attrs[idx] = self.get_type(self.get_pygtk_class_name(cls)).get_object()
             names = names[1:]
+            idx += 1
 
         if names:
-            for i, t in enumerate(gobject.signal_query(signal, str(cls))[-1]):
-                try:
-                    tname = self.get_type(self.get_pygtk_class_name(t.name))
-                    if tname:
-                        attrs[names[i]] = tname.get_object()
-                    else:
-                        attrs[names[i]] = None
-                except IndexError:
-                    break
+            for t in gobject.signal_query(signal, str(cls))[-1]:
+                tname = self.get_type(self.get_pygtk_class_name(t.name))
+                if tname:
+                    attrs[idx] = tname.get_object()
+
+                idx += 1
 
         return attrs
 
@@ -179,7 +175,8 @@ class GladeHandler(xml.sax.handler.ContentHandler):
             self.current_object_class = attrs['class']
         elif name == 'signal':
             if self.current_object_class:
-                self.signals[attrs['handler']] = self.current_object_class, attrs['name']
+                self.signals[attrs['handler']] = (self.current_object_class, attrs['name'],
+                    self._locator.getLineNumber())
 
     def endElement(self, name):
         if name == 'object':
