@@ -29,11 +29,11 @@ def infer_parameter_objects_with_hints(func):
         except AttributeError:
             return params_types
 
-        param_names = pyfunction.get_param_names(False)
-        for i, name in enumerate(param_names):
-            ptype = hintdb.get_function_param_type(pyfunction, name)
-            if ptype is not None:
-                params_types[i] = ptype
+        scope_path = get_attribute_scope_path(pyfunction)
+        provided_names = hintdb.get_function_params(scope_path, pyfunction)
+        for i, t in enumerate(params_types):
+            if i in provided_names:
+                params_types[i] = provided_names[i]
 
         return params_types
 
@@ -50,61 +50,41 @@ def infer_returned_object_with_hints(func):
         except AttributeError:
             return func(pyfunction, args)
 
-        rtype = hintdb.get_function_param_type(pyfunction, 'return')
-        if rtype is None:
-            rtype = func(pyfunction, args)
-
-        return rtype
+        scope_path = get_attribute_scope_path(pyfunction)
+        params = hintdb.get_function_params(scope_path, pyfunction)
+        try:
+            return params['return']
+        except KeyError:
+            return func(pyfunction, args)
 
     return inner
 
 rope.base.oi.soi.infer_returned_object = infer_returned_object_with_hints(
     rope.base.oi.soi.infer_returned_object)
 
-def get_attribute_scope_path(obj):
-    if isinstance(obj, (PyModule, PyPackage)):
-        if obj.resource:
-            return obj.pycore.modname(obj.resource)
+def get_attribute_scope_path(obj, collected=''):
+        if isinstance(obj, (PyModule, PyPackage)):
+            if obj.resource:
+                name = obj.pycore.modname(obj.resource)
+            else:
+                return collected
         else:
-            return ''
-    elif isinstance(obj, (PyClass,)):
-        return get_attribute_scope_path(obj.get_module()) + '.' + obj.get_name()
-    else:
-        return ''
+            name = obj.get_name()
 
-def get_attribute_with_hints(func, what):
-    def inner(self, name):
-        #print what, get_attribute_scope_path(self), name
-
-        getting_name = 'getting_attr_%s' % name
-        if getattr(self, getting_name, False):
-            return func(self, name)
+        if collected:
+            collected = name + '.' + collected
+        else:
+            collected = name
 
         try:
-            hintdb = self.pycore.hintdb
+            scope = obj.get_scope().parent
         except AttributeError:
-            return func(self, name)
-
-        try:
-            setattr(self, getting_name, True)
-            result = hintdb.get_module_attribute(self, name)
-        except exceptions.AttributeNotFoundError:
-            result = None
-        except Exception:
-            raise
-        finally:
-            setattr(self, getting_name, False)
-
-        if result is None:
-            return func(self, name)
-
-        return result
-
-    return inner
-
-PyModule.get_attribute = get_attribute_with_hints(PyModule.get_attribute, 'mod')
-PyPackage.get_attribute = get_attribute_with_hints(PyPackage.get_attribute, 'pkg')
+            return collected
 
+        if scope:
+            return get_attribute_scope_path(scope.pyobject, collected)
+        else:
+            return collected
 
 def get_attributes_with_hints(func):
     def inner(self):
@@ -118,13 +98,26 @@ def get_attributes_with_hints(func):
             return result
 
         scope_path = get_attribute_scope_path(self)
-        result.update(hintdb.get_class_attributes(scope_path, self, result))
+        recursion_guard = '____' + scope_path
+
+        if recursion_guard in self.__dict__:
+            return result
+
+        self.__dict__[recursion_guard] = True
+        try:
+            result.update(hintdb.get_attributes(scope_path, self, result))
+        except:
+            raise
+        finally:
+            del self.__dict__[recursion_guard]
 
         return result
 
     return inner
 
 PyClass._get_structural_attributes = get_attributes_with_hints(PyClass._get_structural_attributes)
+PyModule._get_structural_attributes = get_attributes_with_hints(PyModule._get_structural_attributes)
+PyPackage._get_structural_attributes = get_attributes_with_hints(PyPackage._get_structural_attributes)
 
 
 class HintProvider(object):
@@ -139,35 +132,16 @@ class HintProvider(object):
         """
         return self._project()
 
-    def get_function_param_type(self, pyfunc, name):
-        """Should resolve type for function's parameter `name`
+    def get_function_params(self, scope_path, pyfunc):
+        """Should resolve type for function's parameters `name`
 
-        Also should resolve return type if name == 'return'
-        If there is no any type hints None is returned
+        Also should resolve return type by setting 'return' item in result dict
+        If there is no any type hints {} is returned
         """
-        return None
+        return {}
 
-    def get_scope_path(self, scope):
-        result = []
-        current_scope = scope
-        while current_scope is not None:
-            pyobj = current_scope.pyobject
-            if isinstance(pyobj, PyModule):
-                name = pyobj.pycore.modname(pyobj.resource)
-            else:
-                name = pyobj.get_name()
-
-            result.insert(0, name)
-            current_scope = current_scope.parent
-
-        return '.'.join(result)
-
-    def get_module_attribute(self, pymodule, name):
-        """Resolves module/package attribute's PyName"""
-        return None
-
-    def get_class_attributes(self, scope_path, pyclass, attrs):
-        """Returns additional atributes for pyclass"""
+    def get_attributes(self, scope_path, pyobj, attrs):
+        """Returns additional atributes for pymodule or pyclass"""
         return {}
 
     def get_type(self, type_name, scope=None):
@@ -199,39 +173,23 @@ class ScopeHintProvider(HintProvider):
         super(ScopeHintProvider, self).__init__(project)
         self.matcher = scope_matcher
 
-    def get_function_param_type(self, pyfunc, name):
-        scope_path = self.get_scope_path(pyfunc.get_scope())
-        type_name = self.matcher.find_param_type_for(scope_path, name)
-        if type_name:
-            pyname = self.get_type(type_name)
-            if pyname:
-                return pyname.get_object()
+    def get_function_params(self, scope_path, pyfunc):
+        result = {}
+        for i, name in enumerate(pyfunc.get_param_names(False)+['return']):
+            type_name = self.matcher.find_param_type_for(scope_path, name)
+            if type_name:
+                pyname = self.get_type(type_name)
+                if pyname:
+                    if name == 'return':
+                        result['return'] = pyname.get_object()
+                    else:
+                        result[i] = pyname.get_object()
 
-        return None
+        return result
 
-    def get_module_attribute(self, pymodule, name):
-        scope_path = get_attribute_scope_path(pymodule)
-
-        type_name = self.matcher.find_attribute_type_for(scope_path, name)
-        if type_name:
-            type = self.get_type(type_name)
-        else:
-            type = None
-
-        if type:
-            if type_name.endswith('()'):
-                obj = rope.base.pyobjects.PyObject(type.get_object())
-                pyname = ReplacedName(obj, type)
-            else:
-                pyname = type
-        else:
-            pyname = None
-
-        return pyname
-
-    def get_class_attributes(self, scope_path, pyclass, attrs):
+    def get_attributes(self, scope_path, pyclass, attrs):
         attrs = {}
-        for name, type_name in self.matcher.find_class_attributes(scope_path):
+        for name, type_name in self.matcher.find_attributes(scope_path):
             type = self.get_type(type_name)
             if type:
                 if type_name.endswith('()'):
@@ -245,14 +203,6 @@ class ScopeHintProvider(HintProvider):
 class ScopeMatcher(object):
     """Abstract matcher class for :class:`ScopeHintProvider`"""
 
-    def find_attribute_type_for(self, scope_path, name):
-        """
-        Return matched scope attribute type
-
-        :param scope_path: module name (``program.module.name``)
-        :param name: module attribute name
-        """
-
     def find_param_type_for(self, scope_path, name):
         """
         Return matched function param or return value type
@@ -261,7 +211,7 @@ class ScopeMatcher(object):
         :param name: function param name or `return` in case matching function return value type
         """
 
-    def find_class_attributes(self, scope_path):
+    def find_attributes(self, scope_path):
         """
         Return matched class attributes types
 
@@ -275,35 +225,12 @@ class ReScopeMatcher(ScopeMatcher):
 
     Matching is done with ``re.match`` function (pay attention to differences with ``re.search``)
 
-    See :func:`add_attribute_hint`, :func:`add_param_hint`, :func:`add_class_attribute`.
+    See :func:`add_attribute`, :func:`add_param_hint`.
 
     """
     def __init__(self):
-        self.attribute_hints = []
         self.param_hints = []
-        self.class_attributes = []
-
-    def add_attribute_hint(self, scope, name, object_type):
-        """Add attribute type hint for module scope
-
-        Can be used to provide module attributes types, for example in case or complex
-        module loading (werkzeug) or complex runtime behavior (flask). Here is example
-        ``.ropeproject/ropehints.py``::
-
-            def init(provider):
-                provider.db.add_attribute_hint('flask$', 'request$', 'flask.wrappers.Request()')
-                provider.db.add_attribute_hint('werkzeug$', 'Request$', 'werkzeug.wrappers.Request')
-
-        What happens here? ``flask.request`` is magic proxy to isolate thread contexts from each other.
-        In runtime it is ``flask.wrappers.Request`` object (in default flask setup), so first line
-        adds this missing information. But this is not enough. ``flask.wrappers.Request`` is a child
-        of ``werkzeug.Request`` which can not be resolved because of werkzeug's module
-        loading system. So there is second line adding necessary mapping: module attribute
-        ``werkzeug.Request`` should be ``werkzeug.wrappers.Request`` indeed. Take note about
-        parentheses, ``flask.request`` is instance so type declared with them as opposite to
-        ``werkzeug.Request`` which is class.
-        """
-        self.attribute_hints.append((re.compile(scope), re.compile(name), object_type))
+        self.attributes = []
 
     def add_param_hint(self, scope, name, object_type):
         """Add function/method parameter or return value type hint.
@@ -329,35 +256,45 @@ class ReScopeMatcher(ScopeMatcher):
             provider.db.add_param_hint('re\.compile$', 'return$', 're.RegexObject()')
             provider.db.add_param_hint('re\.search$', 'return$', 're.MatchObject()')
             provider.db.add_param_hint('re\.match$', 'return$', 're.MatchObject()')
-            provider.db.add_attribute_hint('re$', 'RegexObject$', 'snaked.plugins.python.stub.RegexObject')
-            provider.db.add_attribute_hint('re$', 'MatchObject$', 'snaked.plugins.python.stub.MatchObject')
+            provider.db.add_attribute('re$', 'RegexObject', 'snaked.plugins.python.stub.RegexObject')
+            provider.db.add_attribute('re$', 'MatchObject', 'snaked.plugins.python.stub.MatchObject')
 
         Take notice, ``re.compile``, ``re.search``, ``re.match`` return absent classes which are mapped
         with :func:`add_attribute_hint` to existing stubs later.
         """
         self.param_hints.append((re.compile(scope), re.compile(name), object_type))
 
-    def add_class_attribute(self, scope, name, object_type):
-        """Add attribute type hint for class/object attribute
+    def add_attribute(self, scope, name, object_type):
+        """Add attribute type hint for module or class/object
 
-        Example from my django project::
+        Can be used to provide module attributes types, for example in case or complex
+        module loading (werkzeug) or complex runtime behavior (flask). Here is example
+        ``.ropeproject/ropehints.py``::
 
-            provider.db.add_class_attribute('django\.http\.HttpRequest$', 'cur$', 'app.Cur')
-            provider.db.add_class_attribute('django\.http\.HttpRequest$', 'render$', 'app.render')
+            def init(provider):
+                provider.db.add_attribute('flask$', 'request', 'flask.wrappers.Request()')
+                provider.db.add_attribute('werkzeug$', 'Request', 'werkzeug.wrappers.Request')
+
+        What happens here? ``flask.request`` is magic proxy to isolate thread contexts from each other.
+        In runtime it is ``flask.wrappers.Request`` object (in default flask setup), so first line
+        adds this missing information. But this is not enough. ``flask.wrappers.Request`` is a child
+        of ``werkzeug.Request`` which can not be resolved because of werkzeug's module
+        loading system. So there is second line adding necessary mapping: module attribute
+        ``werkzeug.Request`` should be ``werkzeug.wrappers.Request`` indeed. Take note about
+        parentheses, ``flask.request`` is instance so type declared with them as opposite to
+        ``werkzeug.Request`` which is class.
+
+        Also one can add class attributes hint. Here is example from my django project::
+
+            provider.db.add_attribute('django\.http\.HttpRequest$', 'cur', 'app.Cur')
+            provider.db.add_attribute('django\.http\.HttpRequest$', 'render', 'app.render')
 
         Here are some explanations. Every ``request`` object has ``cur`` attribute
         (I know about contexts, don't ask me why I need it) which is instance of ``app.Cur``, so
         first line injects such info. Second line resolves ``render`` function also bounded to
         request.
         """
-        self.class_attributes.append((re.compile(scope), name, object_type))
-
-    def find_attribute_type_for(self, scope_path, name):
-        for scope, vname, otype in self.attribute_hints:
-            if scope.match(scope_path) and vname.match(name):
-                return otype
-
-        return None
+        self.attributes.append((re.compile(scope), name, object_type))
 
     def find_param_type_for(self, scope_path, name):
         for scope, vname, otype in self.param_hints:
@@ -366,8 +303,8 @@ class ReScopeMatcher(ScopeMatcher):
 
         return None
 
-    def find_class_attributes(self, scope_path):
-        for scope, vname, otype in self.class_attributes:
+    def find_attributes(self, scope_path):
+        for scope, vname, otype in self.attributes:
             if scope.match(scope_path):
                 yield vname, otype
 
@@ -394,7 +331,7 @@ class CompositeHintProvider(HintProvider):
     def __init__(self, project):
         super(CompositeHintProvider, self).__init__(project)
 
-        self.class_attributes_cache = {}
+        self.attributes_cache = {}
 
         self.hint_provider = []
 
@@ -405,8 +342,8 @@ class CompositeHintProvider(HintProvider):
         self.db.add_param_hint('re\.compile$', 'return$', 're.RegexObject()')
         self.db.add_param_hint('re\.search$', 'return$', 're.MatchObject()')
         self.db.add_param_hint('re\.match$', 'return$', 're.MatchObject()')
-        self.db.add_attribute_hint('re$', 'RegexObject$', 'snaked.plugins.python.stub.RegexObject')
-        self.db.add_attribute_hint('re$', 'MatchObject$', 'snaked.plugins.python.stub.MatchObject')
+        self.db.add_attribute('re$', 'RegexObject', 'snaked.plugins.python.stub.RegexObject')
+        self.db.add_attribute('re$', 'MatchObject', 'snaked.plugins.python.stub.MatchObject')
 
         self.add_hint_provider(ScopeHintProvider(project, self.db))
 
@@ -422,34 +359,23 @@ class CompositeHintProvider(HintProvider):
         self.hint_provider.insert(0, provider)
         return provider
 
-    def get_function_param_type(self, pyfunc, name):
+    def get_function_params(self, scope_path, pyfunc):
         for p in self.hint_provider:
-            result = p.get_function_param_type(pyfunc, name)
-            if result is not None:
+            result = p.get_function_params(scope_path, pyfunc)
+            if result:
                 return result
 
-        return None
+        return {}
 
-    def get_module_attribute(self, pymodule, name):
-        for p in self.hint_provider:
-            try:
-                result = p.get_module_attribute(pymodule, name)
-                if result is not None:
-                    return result
-            except AttributeError:
-                pass
-
-        return None
-
-    def get_class_attributes(self, scope_path, pyclass, attrs):
+    def get_attributes(self, scope_path, pyclass, attrs):
         try:
-            return self.class_attributes_cache[scope_path]
+            return self.attributes_cache[scope_path]
         except KeyError:
             pass
 
         new_attrs = {}
         for p in self.hint_provider:
-            new_attrs.update(p.get_class_attributes(scope_path, pyclass, attrs))
+            new_attrs.update(p.get_attributes(scope_path, pyclass, attrs))
 
-        self.class_attributes_cache[scope_path] = new_attrs
+        self.attributes_cache[scope_path] = new_attrs
         return new_attrs
\ No newline at end of file
