@@ -11,7 +11,7 @@ import gtk, gobject
 from gtksourceview2 import CompletionProvider, CompletionProposal
 from gtksourceview2 import COMPLETION_ACTIVATION_USER_REQUESTED
 
-from snaked.util import idle
+from snaked.util import idle, join_to_settings_dir
 from .parser import parse_snippets_from
 
 loaded_snippets = {}
@@ -21,6 +21,18 @@ completion_providers = {}
 
 stop_managers = weakref.WeakKeyDictionary()
 
+def init(manager):
+    from snaked.core.prefs import register_dialog
+    register_dialog('Snippets', show_snippet_preferences, 'snippet')
+
+def show_snippet_preferences(editor):
+    if 'not_initialized' in existing_snippet_contexts:
+        existing_snippet_contexts.clear()
+        discover_snippet_contexts()
+
+    from prefs import PreferencesDialog
+    PreferencesDialog(existing_snippet_contexts).show(editor)
+
 def editor_opened(editor):
     if 'not_initialized' in existing_snippet_contexts:
         existing_snippet_contexts.clear()
@@ -29,10 +41,12 @@ def editor_opened(editor):
     if not any(ctx in existing_snippet_contexts for ctx in editor.contexts):
         return
 
+    prior = 50
     contexts = [c for c in editor.contexts if c in existing_snippet_contexts]
     for ctx in contexts:
         if ctx not in loaded_snippets:
-            load_snippets_for(ctx)
+            load_snippets_for(ctx, prior)
+            prior -= 1
 
         editor.view.get_completion().add_provider(completion_providers[ctx])
 
@@ -41,19 +55,20 @@ def editor_opened(editor):
 
     editor.buffer.connect_after('changed', on_buffer_changed)
 
-def load_snippets_for(ctx):
+def load_snippets_for(ctx, prior=None):
     snippets = parse_snippets_from(existing_snippet_contexts[ctx])
     loaded_snippets[ctx] = snippets
     snippet_names = [s.snippet for s in snippets.values()]
     for name in snippet_names:
         snippets_match_hash.setdefault(ctx, {}).setdefault(len(name), {})[name] = True
 
-    completion_providers[ctx] = SnippetsCompletionProvider(ctx)
+    if prior is not None:
+        completion_providers[ctx] = SnippetsCompletionProvider(ctx, prior)
 
 def discover_snippet_contexts():
-    dirs_to_scan = [os.path.join(os.path.dirname(__file__), 'snippets'),
-        os.path.join(os.getenv('XDG_CONFIG_HOME', os.path.expanduser('~/.config')),
-            'snaked', 'snippets')
+    dirs_to_scan = [
+        os.path.join(os.path.dirname(__file__), 'snippets'),
+        join_to_settings_dir('snippets'),
     ]
 
     for d in dirs_to_scan:
@@ -332,20 +347,21 @@ def iter_at_whitespace(iter):
         start.set_line(iter.get_line())
 
         char = start.get_text(iter).decode('utf-8')[-1]
-        return not char.isalnum() and char != '_'
+        return char.isspace() or char in ('>', ')', '}', ']')
 
 
 class SnippetsCompletionProvider(gobject.GObject, CompletionProvider):
-    def __init__(self, ctx):
+    def __init__(self, ctx, priority):
         gobject.GObject.__init__(self)
         self.ctx = ctx
         self.last_editor_ref = None
+        self.priority = priority
 
     def do_get_name(self):
         return '%s snippets' % self.ctx
 
     def do_get_priority(self):
-        return 2
+        return self.priority
 
     def do_set_priority(self):
         pass
